import { isValidElement, cloneElement } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function replaceTree(input, props, index, skipMatch) {
    if (skipMatch === void 0) { skipMatch = false; }
    if (!skipMatch && props.match(input)) {
        return replaceTree(props.replace(input), props, index, true);
    }
    else if (isValidElement(input)) {
        var element = input;
        if (typeof element.type === "function") {
            element = element.type(element.props);
        }
        return cloneElement(element, __assign({}, element.props, { key: index, children: replaceTree(element.props.children, props, index) }));
    }
    else if (Array.isArray(input)) {
        return input.reduce(function (acc, item, index) { return acc.concat(replaceTree(item, props, index)); }, []);
    }
    else {
        return input;
    }
}
function Replacer(props) {
    var match = function () { return false; };
    var replace = props.replace;
    if ("matchElement" in props) {
        match = function (i) { return isValidElement(i) && i.type === props.matchElement; };
    }
    else if ("matchLiteral" in props) {
        match = function (i) {
            return !Array.isArray(i) && !isValidElement(i) && props.matchLiteral(i);
        };
    }
    else {
        match = props.match;
    }
    return replaceTree(props.children, { match: match, replace: replace });
}

export { Replacer };
//# sourceMappingURL=index.es.js.map
